/*****************************************************************************

$Id: em.h 264 2006-10-05 16:33:22Z blackhedd $

File:     ed.h
Date:     06Apr06

Copyright (C) 2006 by Francis Cianfrocca. All Rights Reserved.
Gmail: garbagecat20

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/



#ifdef OS_WIN32
#include "emwin.h"
#endif


// THIS ENTIRE FILE WILL EVENTUALLY BE FOR UNIX BUILDS ONLY.
//#ifdef OS_UNIX

#ifndef __EventMachine__H_
#define __EventMachine__H_

#ifdef BUILD_FOR_RUBY
  #include <ruby.h>
  #define EmSelect rb_thread_select
#else
  #define EmSelect select
#endif


#ifdef OS_UNIX
typedef long long Int64;
#endif
#ifdef OS_WIN32
typedef __int64 Int64;
#endif

extern time_t gCurrentLoopTime;

class EventableDescriptor;


/********************
class EventMachine_t
********************/

class EventMachine_t
{
	public:
		EventMachine_t (void(*event_callback)(const char*, int, const char*, int));
		virtual ~EventMachine_t();

		void Run();
		void ScheduleHalt();
		void SignalLoopBreaker();
		const char *InstallOneshotTimer (int);
		const char *ConnectToServer (const char *, int);
		const char *CreateTcpServer (const char *, int);
		const char *OpenDatagramSocket (const char *, int);
		const char *CreateUnixDomainServer (const char*);
		const char *_OpenFileForWriting (const char*);

		void Add (EventableDescriptor*);

		void SetTimerQuantum (int);

	public:
		enum { // Event names
			TIMER_FIRED = 100,
			CONNECTION_READ = 101,
			CONNECTION_UNBOUND = 102,
			CONNECTION_ACCEPTED = 103,
			CONNECTION_COMPLETED = 104,
			LOOPBREAK_SIGNAL = 105
		};


	private:
		bool _RunOnce();
		bool _RunTimers();
		void _AddNewDescriptors();
		void _InitializeLoopBreaker();
		void _ReadLoopBreaker();

	private:
		enum {
			MaxOutstandingTimers = 40,
			HeartbeatInterval = 2
		};
		void (*EventCallback)(const char*, int, const char*, int);

		class Timer_t: public Bindable_t {
		};

		multimap<Int64, Timer_t> Timers;
		vector<EventableDescriptor*> Descriptors;
		vector<EventableDescriptor*> NewDescriptors;

		time_t NextHeartbeatTime;

		int LoopBreakerReader;
		int LoopBreakerWriter;
		#ifdef OS_WIN32
		struct sockaddr_in LoopBreakerTarget;
		#endif

		timeval Quantum;
};




#endif // __EventMachine__H_

//#endif // OS_UNIX
