/*****************************************************************************

$Id: rubymain.cpp 265 2006-10-05 19:07:45Z blackhedd $

File:     libmain.cpp
Date:     06Apr06

Copyright (C) 2006 by Francis Cianfrocca. All Rights Reserved.
Gmail: garbagecat20

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "project.h"
#include "eventmachine.h"
#include <ruby.h>



/*******
Statics
*******/

static VALUE EmModule;


/****************
t_event_callback
****************/

static void event_callback (const char *a1, int a2, const char *a3, int a4)
{
	rb_funcall (EmModule, rb_intern ("event_callback"), 3, rb_str_new2(a1), (a2 << 1) | 1, rb_str_new(a3,a4));
}



/**************************
t_initialize_event_machine
**************************/

static VALUE t_initialize_event_machine (VALUE self)
{
	evma_initialize_library (event_callback);
	return Qnil;
}



/*****************************
t_run_machine_without_threads
*****************************/

static VALUE t_run_machine_without_threads (VALUE self)
{
	evma_run_machine();
	return Qnil;
}


/*******************
t_add_oneshot_timer
*******************/

static VALUE t_add_oneshot_timer (VALUE self, VALUE interval)
{
	const char *f = evma_install_oneshot_timer (FIX2INT (interval));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "no timer");
	return rb_str_new2 (f);
}


/**************
t_start_server
**************/

static VALUE t_start_server (VALUE self, VALUE server, VALUE port)
{
	const char *f = evma_create_tcp_server (StringValuePtr(server), FIX2INT(port));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "no acceptor");
	return rb_str_new2 (f);
}


/*******************
t_start_unix_server
*******************/

static VALUE t_start_unix_server (VALUE self, VALUE filename)
{
	const char *f = evma_create_unix_domain_server (StringValuePtr(filename));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "no unix-domain acceptor");
	return rb_str_new2 (f);
}



/***********
t_send_data
***********/

static VALUE t_send_data (VALUE self, VALUE signature, VALUE data, VALUE data_length)
{
	int b = evma_send_data_to_connection (StringValuePtr (signature), StringValuePtr (data), FIX2INT (data_length));
	return INT2NUM (b);
}


/***********
t_start_tls
***********/

static VALUE t_start_tls (VALUE self, VALUE signature)
{
	evma_start_tls (StringValuePtr (signature));
	return Qnil;
}

/**************
t_get_peername
**************/

static VALUE t_get_peername (VALUE self, VALUE signature)
{
	struct sockaddr s;
	if (evma_get_peername (StringValuePtr (signature), &s)) {
		return rb_str_new ((const char*)&s, sizeof(s));
	}

	return Qnil;
}

/*****************************
t_get_comm_inactivity_timeout
*****************************/

static VALUE t_get_comm_inactivity_timeout (VALUE self, VALUE signature)
{
	int timeout;
	if (evma_get_comm_inactivity_timeout (StringValuePtr (signature), &timeout))
		return INT2FIX (timeout);
	return Qnil;
}

/*****************************
t_set_comm_inactivity_timeout
*****************************/

static VALUE t_set_comm_inactivity_timeout (VALUE self, VALUE signature, VALUE timeout)
{
	int ti = FIX2INT (timeout);
	if (evma_set_comm_inactivity_timeout (StringValuePtr (signature), &ti));
		return Qtrue;
	return Qnil;
}


/***************
t_send_datagram
***************/

static VALUE t_send_datagram (VALUE self, VALUE signature, VALUE data, VALUE data_length, VALUE address, VALUE port)
{
	int b = evma_send_datagram (StringValuePtr (signature), StringValuePtr (data), FIX2INT (data_length), StringValuePtr(address), FIX2INT(port));
	return INT2NUM (b);
}


/******************
t_close_connection
******************/

static VALUE t_close_connection (VALUE self, VALUE signature, VALUE after_writing)
{
	evma_close_connection (StringValuePtr (signature), ((after_writing == Qtrue) ? 1 : 0));
	return Qnil;
}



/****************
t_connect_server
****************/

static VALUE t_connect_server (VALUE self, VALUE server, VALUE port)
{
	const char *f = evma_connect_to_server (StringValuePtr(server), FIX2INT(port));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "no connection");
	return rb_str_new2 (f);
}

/*****************
t_open_udp_socket
*****************/

static VALUE t_open_udp_socket (VALUE self, VALUE server, VALUE port)
{
	const char *f = evma_open_datagram_socket (StringValuePtr(server), FIX2INT(port));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "no datagram socket");
	return rb_str_new2 (f);
}



/*****************
t_release_machine
*****************/

static VALUE t_release_machine (VALUE self)
{
	evma_release_library();
	return Qnil;
}


/******
t_stop
******/

static VALUE t_stop (VALUE self)
{
	evma_stop_machine();
	return Qnil;
}

/******************
t_signal_loopbreak
******************/

static VALUE t_signal_loopbreak (VALUE self)
{
	evma_signal_loopbreak();
	return Qnil;
}

/**************
t_library_type
**************/

static VALUE t_library_type (VALUE self)
{
	return rb_eval_string (":extension");
}



/*******************
t_set_timer_quantum
*******************/

static VALUE t_set_timer_quantum (VALUE self, VALUE interval)
{
  evma_set_timer_quantum (FIX2INT (interval));
  return Qnil;
}



/*************
t__write_file
*************/

static VALUE t__write_file (VALUE self, VALUE filename)
{
	const char *f = evma__write_file (StringValuePtr (filename));
	if (!f || !*f)
		rb_raise (rb_eRuntimeError, "file not opened");
	return rb_str_new2 (f);
}


/*********************
Init_rubyeventmachine
*********************/

extern "C" void Init_rubyeventmachine()
{
	// INCOMPLETE, we need to define class Connections inside module EventMachine
	// run_machine and run_machine_without_threads are now identical.
	// Must deprecate the without_threads variant.
	EmModule = rb_define_module ("EventMachine");
	rb_define_module_function (EmModule, "initialize_event_machine", (VALUE(*)(...))t_initialize_event_machine, 0);
	rb_define_module_function (EmModule, "run_machine", (VALUE(*)(...))t_run_machine_without_threads, 0);
	rb_define_module_function (EmModule, "run_machine_without_threads", (VALUE(*)(...))t_run_machine_without_threads, 0);
	rb_define_module_function (EmModule, "add_oneshot_timer", (VALUE(*)(...))t_add_oneshot_timer, 1);
	rb_define_module_function (EmModule, "start_tcp_server", (VALUE(*)(...))t_start_server, 2);
	rb_define_module_function (EmModule, "start_unix_server", (VALUE(*)(...))t_start_unix_server, 1);
	rb_define_module_function (EmModule, "start_tls", (VALUE(*)(...))t_start_tls, 1);
	rb_define_module_function (EmModule, "send_data", (VALUE(*)(...))t_send_data, 3);
	rb_define_module_function (EmModule, "send_datagram", (VALUE(*)(...))t_send_datagram, 5);
	rb_define_module_function (EmModule, "close_connection", (VALUE(*)(...))t_close_connection, 2);
	rb_define_module_function (EmModule, "connect_server", (VALUE(*)(...))t_connect_server, 2);
	rb_define_module_function (EmModule, "open_udp_socket", (VALUE(*)(...))t_open_udp_socket, 2);
	rb_define_module_function (EmModule, "release_machine", (VALUE(*)(...))t_release_machine, 0);
	rb_define_module_function (EmModule, "stop", (VALUE(*)(...))t_stop, 0);
	rb_define_module_function (EmModule, "signal_loopbreak", (VALUE(*)(...))t_signal_loopbreak, 0);
	rb_define_module_function (EmModule, "library_type", (VALUE(*)(...))t_library_type, 0);
	rb_define_module_function (EmModule, "set_timer_quantum", (VALUE(*)(...))t_set_timer_quantum, 1);

  // Provisional:
	rb_define_module_function (EmModule, "_write_file", (VALUE(*)(...))t__write_file, 1);

	rb_define_module_function (EmModule, "get_peername", (VALUE(*)(...))t_get_peername, 1);
	rb_define_module_function (EmModule, "get_comm_inactivity_timeout", (VALUE(*)(...))t_get_comm_inactivity_timeout, 1);
	rb_define_module_function (EmModule, "set_comm_inactivity_timeout", (VALUE(*)(...))t_set_comm_inactivity_timeout, 2);

	rb_define_class_under (EmModule, "ConnectionNotBound", rb_eException);
	rb_define_class_under (EmModule, "NoHandlerForAcceptedConnection", rb_eException);
	rb_define_class_under (EmModule, "UnknownTimerFired", rb_eException);

	rb_define_const (EmModule, "TimerFired", INT2NUM(100));
	rb_define_const (EmModule, "ConnectionData", INT2NUM(101));
	rb_define_const (EmModule, "ConnectionUnbound", INT2NUM(102));
	rb_define_const (EmModule, "ConnectionAccepted", INT2NUM(103));
	rb_define_const (EmModule, "ConnectionCompleted", INT2NUM(104));
	rb_define_const (EmModule, "LoopbreakSignalled", INT2NUM(105));
}

