# $Id: deferrable.rb 216 2006-07-17 10:20:42Z blackhedd $
#
# Author:: blackhedd (gmail address: garbagecat10).
# Date:: 16 July 2006
# 
# Copyright (C) 2006 by Francis Cianfrocca. All Rights Reserved.
#
# This program is made available under the terms of the GPL version 2.
#
# See EventMachine and EventMachine::Connection for documentation and
# usage examples.
#
#----------------------------------------------------------------------------
#
# Copyright (C) 2006 by Francis Cianfrocca. All Rights Reserved.
#
# Gmail: garbagecat10
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#---------------------------------------------------------------------------
#
# 

require 'forwardable'

module EventMachine

module Deferrable
  def callback &block
    return unless block
    if @deferred_status == :succeeded
      block.call
    else
      @callbacks ||= []
      @callbacks << block
    end
  end

  def errback &block
    return unless block
    if @deferred_status == :failed
      block.call
    else
      @errbacks ||= []
      @errbacks << block
    end
  end

  # Note that if you call this method without arguments,
  # no arguments will be passed to the callback/errback.
  # If the user has coded these with arguments, then the
  # user code will throw an argument exception.
  # Implementors of deferrable classes <b>must</b>
  # document the arguments they will supply to user callbacks.
  def set_deferred_status arg, *args
    @deferred_status = arg
    @deferred_args = args
    case @deferred_status
    when :succeeded
      if @callbacks
        while cb = @callbacks.shift
          cb.call *@deferred_args
        end
      end
    when :failed
      if @errbacks
        while eb = @errbacks.shift
          eb.call *@deferred_args
        end
      end
    end
  end

end
end

